# Copyright 2020 Adap GmbH. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""DefaultStrategy tests."""


from .default import DefaultStrategy


def test_default_strategy_num_fit_clients_20_available() -> None:
    """Test num_fit_clients function."""

    # Prepare
    strategy = DefaultStrategy()
    expected = 2

    # Execute
    actual, _ = strategy.num_fit_clients(num_available_clients=20)

    # Assert
    assert expected == actual


def test_default_strategy_num_fit_clients_19_available() -> None:
    """Test num_fit_clients function."""

    # Prepare
    strategy = DefaultStrategy()
    expected = 2

    # Execute
    actual, _ = strategy.num_fit_clients(num_available_clients=19)

    # Assert
    assert expected == actual


def test_default_strategy_num_fit_clients_10_available() -> None:
    """Test num_fit_clients function."""

    # Prepare
    strategy = DefaultStrategy()
    expected = 2

    # Execute
    actual, _ = strategy.num_fit_clients(num_available_clients=10)

    # Assert
    assert expected == actual


def test_default_strategy_num_fit_clients_minimum() -> None:
    """Test num_fit_clients function."""

    # Prepare
    strategy = DefaultStrategy()
    expected = 2

    # Execute
    actual, _ = strategy.num_fit_clients(num_available_clients=9)

    # Assert
    assert expected == actual


def test_default_strategy_num_evaluation_clients_40_available() -> None:
    """Test num_evaluation_clients function."""

    # Prepare
    strategy = DefaultStrategy(fraction_eval=0.05)
    expected = 2

    # Execute
    actual, _ = strategy.num_evaluation_clients(num_available_clients=40)

    # Assert
    assert expected == actual


def test_default_strategy_num_evaluation_clients_39_available() -> None:
    """Test num_evaluation_clients function."""

    # Prepare
    strategy = DefaultStrategy(fraction_eval=0.05)
    expected = 2

    # Execute
    actual, _ = strategy.num_evaluation_clients(num_available_clients=39)

    # Assert
    assert expected == actual


def test_default_strategy_num_evaluation_clients_20_available() -> None:
    """Test num_evaluation_clients function."""

    # Prepare
    strategy = DefaultStrategy(fraction_eval=0.05)
    expected = 2

    # Execute
    actual, _ = strategy.num_evaluation_clients(num_available_clients=20)

    # Assert
    assert expected == actual


def test_default_strategy_num_evaluation_clients_minimum() -> None:
    """Test num_evaluation_clients function."""

    # Prepare
    strategy = DefaultStrategy(fraction_eval=0.05)
    expected = 2

    # Execute
    actual, _ = strategy.num_evaluation_clients(num_available_clients=19)

    # Assert
    assert expected == actual
